@@ -2,12 +2,12 @@ from typing import Annotated, Any
 
 from pydantic import BaseModel, Field
 
-from unique_toolkit._common.default_language_model import DEFAULT_GPT_4o
 from unique_toolkit._common.validators import LMI, get_LMI_default_field
 from unique_toolkit.agentic.evaluation.context_relevancy.schema import (
     StructuredOutputConfig,
 )
 from unique_toolkit.agentic.tools.config import get_configuration_dict
+from unique_toolkit.language_model.default_language_model import DEFAULT_GPT_4o
 
 
 class ChunkRelevancySortConfig(BaseModel):
