@@ -2,6 +2,7 @@ from datetime import datetime
 from enum import StrEnum
 from typing import Any, Optional
 
+import unique_sdk
 from humps import camelize
 from pydantic import BaseModel, ConfigDict, Field
 
@@ -92,6 +93,7 @@ class Content(BaseModel):
     updated_at: datetime | None = None
     metadata: dict[str, Any] | None = None
     ingestion_config: dict | None = None
+    ingestion_state: str | None = None
 
 
 class ContentReference(BaseModel):
@@ -114,6 +116,22 @@ class ContentReference(BaseModel):
         description="List of indices in the ChatMessage original_content this reference refers to. This is usually the id in the functionCallResponse. List type due to implementation in node-chat",
     )
 
+    @classmethod
+    def from_sdk_reference(
+        cls, reference: unique_sdk.Message.Reference | unique_sdk.Space.Reference
+    ) -> "ContentReference":
+        kwargs = {
+            "name": reference["name"],
+            "url": reference["url"],
+            "sequence_number": reference["sequenceNumber"],
+            "source": reference["source"],
+            "source_id": reference["sourceId"],
+        }
+        if "originalIndex" in reference:
+            kwargs["original_index"] = reference["originalIndex"]
+
+        return cls.model_validate(kwargs)
+
 
 class ContentSearchType(StrEnum):
     COMBINED = "COMBINED"
@@ -176,15 +194,19 @@ class PaginatedContentInfos(BaseModel):
     total_count: int
 
 
-class FolderInfo(BaseModel):
+class BaseFolderInfo(BaseModel):
     model_config = model_config
     id: str
     name: str
+    parent_id: str | None
+
+
+class FolderInfo(BaseFolderInfo):
+    model_config = model_config
     ingestion_config: dict[str, Any]
-    createdAt: str | None
-    updatedAt: str | None
-    parentId: str | None
-    externalId: str | None
+    created_at: str | None
+    updated_at: str | None
+    external_id: str | None
 
 
 class DeleteContentResponse(BaseModel):
